<!DOCTYPE html>
<html>
  {% include "head.html.j2" %}
  <body>
    {% include "nav.html" %}
    <main class="responsive">
      {% include "analysis.html.j2" %}
      {% include "results.html.j2" %}
      {% include "external.html.j2" %}

      <div id="export-report" class="page padding">
        <h5>Export report</h5>
        <p>Here will be an option to export the report, either as a stand alone html page or a PDF document</p>
      </div>

      {% include "documentation.html" %}

    </main>
  </body>
</html>

