<div id="documentation" class="page padding">
  <h5>Documentation</h5>
  <h6>Introduction</h6>
  <p>The Genetic Therapy Generator Toolkit helps users determine whether a given variant (or set of variants) can be corrected with a genetic therapy. Rather than relying on explicit guidelines about which kinds of variants can be treated, GTGT simulates the effect of every possible therapy and ranks the therapies based on how many of the transcript features are restored.</p>
  <p>Because of this unique approach, GTGT can identify possible therapies when it appears none are available, for example by skipping upstream or downstream exons to restore the reading frame, or by skipping adjacent exons (experimental).</p>
  <h6>Interpreting results</h6>
  <p>The first two entries on the results page will always be the analysis of the <em>Wildtype</em> and <em>Input</em> versions of the transcript. Each entry contains the name, HGVS notation and short description of the therapy. The most important section is the <em>Annotation overview</em>, which lists the percentage of each feature of the transcript that remains after the mutations (and therapy) have been applied.

  The <em>Exons</em> feature refers to all bases that make up the exon. When GTGT skips an exon, the bases of the exon will be removed from the <em>Exons</em> feature, which can be seen from the percentage and the remaining basepairs.

  The <em>Coding exons</em> refer to the bases that make up the coding sequence of the transcript. Changes in the <em>Coding exons</em> take the actual predicted protein changes in the transcript into account. For example, in the case of a frameshift variant all bases after the frameshift will be removed from the <em>Coding exons</em> feature, but not from the <em>Exons</em>.
  <p>The extended documentation for this tool can be found on <a class="link" href="https://gtgt.readthedocs.io/en/latest/">https://gtgt.readthedocs.io</a>.</p>
</div>

<div id="development" class="page padding">
  <h5>Development</h5>
  <p>The Genetic Therapy Generator Toolkit has been developed by the <a href="https://rnatherapy.nl/", class="link", target="_blank">Dutch Center for RNA Therapeutics</a>, who have committed to continue to fund the development of GTGT until at least June 2027.</p>
  <p>GTGT can be installed locally via <a class="link" target="_blank" href="https://pypi.org/p/gtgt">PyPi</a>, or you can install the development version from <a class="link" href="https://github.com/DCRT-LUMC/GTGT" target="_blank">Github</a>. We welcome contributions, bug reports and other feedback on GitHub or via an email to <a class="link" href="mailto:DCRT@LUMC.nl">DCRT@LUMC.nl</a>.</p>
  <p>GTGT is free to use and redistribute under the conditions of the <em>GNU Affero General Public License v3.0</em>.</p>
</div>

<div id="disclaimer" class="page padding">
  <h5>Disclaimer</h5>
  <p>Copyright© 2023 LUMC <a class="link" href="https://www.lumc.nl">https://www.lumc.nl</a></p>
  <p>This program is free software: you can redistribute it and/or modify it under the terms of the GNU Affero General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.</p>
  <p>This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more details.</p>
  <p>By accessing and using the program in any manner (including copying, modifying or redistributing the program), you accept and agree to the applicability of the GNU Affero General Public License. You can find and read this license on GNU Affero General Public License - GNU Project - Free Software Foundation.</p>
</div>
